import os
from argparse import Namespace
from pathlib import Path

from luscious_dl.logger import logger_file_handler, logger
from luscious_dl.parser import is_a_valid_integer
from luscious_dl.start import start
from luscious_dl.utils import cls, create_default_files, open_config_menu, get_config_data, read_list, info, \
  ListFilesManager


def list_txt_organizer(items: list[str], prefix: str) -> None:
  """
  Remove from list.txt and then add to list_completed.txt
  :param items: List of urls or ids
  :param prefix: album/user
  """
  for item in items:
    ListFilesManager.remove(item)
    ListFilesManager.add(f'{prefix}-{int(item)}' if is_a_valid_integer(item) else item)


def create_namespace(album_inputs=None, user_inputs=None, keyword=None, search_download=False, page=1, max_pages=1,
                     output_dir=None, threads=os.cpu_count() or 1, retries=5, timeout=30, delay=0,
                     sorting='date_trending', foldername_format='%t', only_favorites=False, gen_pdf=False,
                     rm_origin_dir=False) -> Namespace:
  return Namespace(album_inputs=album_inputs, user_inputs=user_inputs, keyword=keyword, search_download=search_download,
                   page=page, max_pages=max_pages, output_dir=output_dir, threads=threads, retries=retries,
                   timeout=timeout, delay=delay, sorting=sorting, foldername_format=foldername_format,
                   only_favorites=only_favorites, gen_pdf=gen_pdf, rm_origin_dir=rm_origin_dir)


def menu() -> None:
  """Menu"""
  info()
  create_default_files()
  logger_file_handler()
  configs = get_config_data()
  output_dir = Path(os.path.normcase(configs.get('directory', './Albums/'))).resolve()
  pool_size = configs.get('pool', 1)
  retries = configs.get('retries', 5)
  timeout = configs.get('timeout', 30)
  delay = configs.get('delay', 0)
  foldername_format = configs.get('foldername_format', '%t')
  gen_pdf = configs.get('gen_pdf', False)
  rm_origin_dir = configs.get('rm_origin_dir', False)

  while True:
    option = input('Options:\n'
                   '1 - Download albums by URL or ID.\n'
                   '2 - Download all user albums.\n'
                   '3 - Download all user favorites.\n'
                   '4 - Search albums by keyword.\n'
                   '5 - Download albums from list.txt.\n'
                   '6 - Settings.\n'
                   '0 - Exit.\n'
                   '> ')

    if option in ('1', '2', '3'):
      inputs = input('\n0 - Back.\n'
                     f'Enter {"album" if option == "1" else "user"} URL or ID.\n> ')
      cls()
      if inputs != '0':
        args = create_namespace(album_inputs=inputs if option == '1' else None,
                                user_inputs=inputs if option in ('2', '3') else None,
                                output_dir=output_dir, threads=pool_size, retries=retries, timeout=timeout, delay=delay,
                                foldername_format=foldername_format,
                                only_favorites=option == '3',
                                gen_pdf=gen_pdf, rm_origin_dir=rm_origin_dir)
        start(args)
        list_txt_organizer([input_.strip() for input_ in inputs.split(',')], 'album' if option == '1' else 'user')
        logger.log(5, 'URLs/IDs added to completed list.')

    elif option == '4':
      keyword = input('Enter keyword\n> ')
      if not keyword:
        print('Please enter a keyword.\n')
        return
      page = input('Enter starting page number or leave blank\n> ')
      page = int(page) if is_a_valid_integer(page) else 1
      max_pages = input('Enter max page or leave blank\n> ')
      max_pages = int(max_pages) if is_a_valid_integer(max_pages) else 1
      search_download = input('Download search results? ("Y/N") ').strip() in 'yY'
      args = create_namespace(keyword=keyword, search_download=search_download, page=page, max_pages=max_pages)
      start(args)

    elif option == '5':
      list_txt = list(set(read_list()))
      args = create_namespace(album_inputs=','.join(list_txt),
                              output_dir=output_dir,
                              threads=pool_size, retries=retries, timeout=timeout, delay=delay)
      start(args)
      list_txt_organizer(list_txt, 'album')
      logger.log(5, 'URLs/IDs added to completed list.')

    elif option == '6':
      open_config_menu()

    elif option == '0':
      exit()

    else:
      print('Invalid Option.\n')


if __name__ == '__main__':
  menu()



import json
import os
import re
import shutil
from pathlib import Path
from typing import Optional

from luscious_dl import __version__
from luscious_dl.logger import logger


def info() -> None:
  """Show package version."""
  logger.info(f'Luscious Downloader version: {__version__}')


def cls() -> None:
  """Clears the command prompt."""
  os.system('cls' if os.name == 'nt' else 'clear')


"""def format_filename(name: str) -> str:
  pass"""


def format_foldername(album, foldername_format: str = '%t') -> str:
  """
  Format album folder name.
  :param album: Album instance
  :param foldername_format:
    %i = album id
    %t = album name
    %a = album author
    %p = album pictures
    %g = album gifs
  :return: formatted folder name string
  """
  album_name = re.sub(r'[^\w\-_\. ]', '_', album.title)
  folder_name = foldername_format \
      .replace('%i', str(album.id_)) \
      .replace('%t', album_name) \
      .replace('%a', album.author) \
      .replace('%p', str(album.number_of_pictures)) \
      .replace('%g', str(album.number_of_animated_pictures)) \
      .replace('[]', '').strip()
  return folder_name


def create_folder(directory: Path) -> None:
  """
  Creates folder in the specified path.
  :param directory: folder path
  """
  try:
    if not Path.exists(directory):
      Path.mkdir(directory, exist_ok=True)
      logger.info(f'Album folder created in: {directory}')
    else:
      logger.warn(f'Album folder already exist in: {directory}')
  except Exception as e:
    logger.error(f'Create folder: {e}')


def generate_pdf(output_dir: Path, formmatted_name: str, album_folder: Path, rm_origin_dir=False) -> None:
  """
  Create pdf file containing album pictures [jpg,jpeg].
  :param output_dir: output folder path
  :param formmatted_name: formmatted album name
  :param album_folder: album folder path
  :param rm_origin_dir: indicates whether the source folder will be deleted
  """
  try:
    from PIL import Image
    logger.info('Generating album pdf file...')

    pictures_path_list = []
    for file_name in album_folder.iterdir():
      if file_name.suffix.lower() not in ['.jpg', '.jpeg', '.png']:
        continue
      picture_path = Path.joinpath(album_folder, file_name)
      if picture_path.is_dir():
        continue
      pictures_path_list.append(picture_path)

    pictures = []
    if len(pictures_path_list) > 0:
      for picture_path in pictures_path_list:
        img = Image.open(picture_path)
        if picture_path.suffix.lower() == '.png':
          img = img.convert('RGB')
        pictures.append(img)

    if len(pictures) == 0:
      raise Exception('Pictures list is empty, probably has no valid images [jpg, jpeg, png]')

    pdf_filename = f'{formmatted_name}.pdf'
    pdf_path = Path.joinpath(output_dir, pdf_filename)

    logger.info(f'Adding {len(pictures)} pictures to pdf...')
    first_pic = pictures[0]
    pictures.pop(0)

    first_pic.save(pdf_path, save_all=True, append_images=pictures)
    logger.log(5, f'Album PDF saved to: {output_dir}')

    if rm_origin_dir:
      shutil.rmtree(album_folder, ignore_errors=True)
      logger.log(5, f'Album {formmatted_name} folder deleted.')

  except ImportError:
    logger.error('Please install Pillow package by using pip.')
  except Exception as e:
    logger.error(f'Failed to generate album pdf: {e}')


# \/ Menu only functions \/ #

def get_root_path() -> Path:
  """
  Return project root path.
  :return: PurePath subclass
  """
  return Path(__file__).parent.parent


def get_config_data() -> Optional[dict]:
  """
  Load and return config.json data.
  :return: key value
  """
  try:
    with get_root_path().joinpath('config.json').open() as config:
      data = json.load(config)
      return data
  except Exception as e:
    logger.warning(f'Something went wrong loading config file: {e}')
    return None


def read_list() -> list[str]:
  """
  Read list.txt file content.
  :return: list.txt content
  """
  try:
    logger.log(5, 'Reading list...')
    with get_root_path().joinpath('list.txt').open() as list_file:
      list_txt = list_file.read()
      if len(list_txt) > 0:
        list_txt = list_txt.split('\n')
      logger.log(5, f'Total of Items: {len(list_txt)}.')
    return list_txt
  except Exception as e:
    print(f'Failed to read the list.txt.\n{e}')


def create_default_files() -> None:
  """Create the initial files when using the menu."""
  root = get_root_path()
  if not Path.exists(root.joinpath('config.json')):
    data = {
      "directory": os.path.normcase("./Albums/"),
      "pool": os.cpu_count() or 1,
      "retries": 5,
      "timeout": 30,
      "delay": 0,
      "foldername_format": "%t",
      "gen_pdf": False,
      "rm_origin_dir": False
    }
    with root.joinpath('config.json').open('a+') as config_file:
      json.dump(data, config_file, indent=2)
  if not Path.exists(root.joinpath('list.txt')):
    root.joinpath('list.txt').touch()
  if not Path.exists(root.joinpath('list_completed.txt')):
    root.joinpath('list_completed.txt').touch()


class ListFilesManager:
  """Class to manage url list files."""
  @staticmethod
  def add(string: str) -> None:
    """
    Add string to list_completed.txt
    :param string: Mostly URL or ID of Album or User
    """
    path = get_root_path().joinpath('list_completed.txt')
    with path.open() as completed:
      text = completed.read()
    with path.open('a') as completed:
      if not text.endswith('\n'):
        completed.write('\n')
      completed.write(string)
      logger.log(5, f'Added to completed list: {string}')

  @staticmethod
  def remove(string: str) -> None:
    """
    Remove string from list.txt
    :param string: Mostly URL or ID of Album or User
    """
    path = get_root_path().joinpath('list.txt')
    with path.open() as list_txt:
      temp = ['' if string in line else line for line in list_txt]
    with path.open('w') as list_txt:
      for line in temp:
        list_txt.write(line)


def open_config_menu() -> None:
  """Open settings/config menu"""
  with get_root_path().joinpath('config.json').open('r+') as json_file:
    data = json.load(json_file)
    while True:
      config_menu = input(f'1 - Change Directory [Current: {data.get("directory")}]\n'
                          f'2 - CPU Pool [Current: {data.get("pool")}]\n'
                          f'3 - Picture Retries [Current: {data.get("retries")}]\n'
                          f'4 - Picture Timeout [Current: {data.get("timeout")}]\n'
                          f'5 - Download Delay [Current: {data.get("delay")}]\n'
                          f'6 - Format output album folder name [Current: {data.get("foldername_format")}]\n'
                          f'7 - Generate PDF: [Current: {data.get("gen_pdf")}]\n'
                          f'8 - Remove origin directory [Current: {data.get("rm_origin_dir")}]\n  '
                          '0 - Back.\n'
                          '> ')
      cls()
      if config_menu == '1':
        new_path = input(f'Current directory: {data.get("directory")}\n'
                         '1 - Restore default directory\n'
                         '0 - Back\n'
                         'Directory: ')
        if new_path not in ['0', '1', ' ']:
          data['directory'] = os.path.normcase(new_path)
        elif new_path == '1':
          data['directory'] = './Albums/'
      elif config_menu == '2':
        print(f'You have: {os.cpu_count()} cores.')
        data['pool'] = int(input('Enter CPU Pool for download pictures.\n> '))
      elif config_menu == '3':
        data['retries'] = int(input('Enter how many pictures retries.\n> '))
      elif config_menu == '4':
        data['timeout'] = int(input('Enter picture timeout.\n> '))
      elif config_menu == '5':
        data['delay'] = int(input('Enter album download delay.\n> '))
      elif config_menu == '6':
        print(
          'Supported album folder formatter:',
          '%i = Album ID',
          '%t = Album name/title',
          '%a = Album authors name',
          '%p = Album total pictures',
          '%g = Album total gifs',
          sep='\n'
        )
        new_folderformat = input('\nEnter album folder format.\n> ')
        if not any(identifier in new_folderformat for identifier in ('%i', '%t', '%a')):
          if input('\nNo album identifiers found.\nYou sure? ("Y/N")\n> ') in 'nN':
            print('\nAlbum folder format set to the default.\n')
            new_folderformat = '%t'
        data['foldername_format'] = new_folderformat
      elif config_menu == '7':
        data['gen_pdf'] = not data.get('gen_pdf')
      elif config_menu == '8':
        data['rm_origin_dir'] = not data.get('rm_origin_dir')
      elif config_menu == '0':
        cls()
        break
      else:
        print('Invalid Option.\n')
      json_file.seek(0)
      json.dump(data, json_file, indent=2)
      json_file.truncate()



import argparse
import os
from pathlib import Path


def command_line() -> argparse.Namespace:
  """
  Defines the command line interface.
  :return: arguments
  """
  parser = argparse.ArgumentParser(prog='Luscious Downloader', description='Download albums')

  group = parser.add_mutually_exclusive_group(required=True)

  group.add_argument('--album', '-a', dest='album_inputs', action='store', help='download album by url or id')

  group.add_argument('--user', '-u', dest='user_inputs', action='store', help='download all user albums by url or id')

  group.add_argument('--search', '-s', dest='keyword', action='store', help='search albums by keyword')

  # user args
  parser.add_argument('--favorites', '-f', dest='only_favorites', action='store_true',
                      help="download only user favorites")

  # search args
  parser.add_argument('--download', '-d', dest='search_download', action='store_true',
                      help='download albums from search results')

  parser.add_argument('--page', dest='page', action='store', type=int, default=1, help='page number of search results')

  parser.add_argument('--max-page', dest='max_pages', action='store', type=int, default=1,
                      help='max pages of search results')

  parser.add_argument('--sorting', dest='sorting', action='store', default='date_trending',
                      help='search sorting', choices=['date_trending', 'rating_all_time'])

  # generate options
  parser.add_argument('--pdf', '-p', dest='gen_pdf', action='store_true',
                      help='enable pdf generation')
  parser.add_argument('--rm-origin-dir', dest='rm_origin_dir', action='store_true',
                      help='remove downloaded album dir when generated PDF file.')

  # downloader args
  parser.add_argument('--output', '-o', dest='output_dir', action='store', default=Path.cwd(), help='output directory')

  parser.add_argument('--threads', '-t', dest='threads', action='store', type=int, default=os.cpu_count(),
                      help='how many download threads')

  parser.add_argument('--retries', '-R', dest='retries', action='store', type=int, default=5, help='download retries')

  parser.add_argument('--timeout', '-T', dest='timeout', action='store', type=int, default=30, help='download timeout')

  parser.add_argument('--delay', '-D', dest='delay', action='store', type=int, default=0,
                      help='delay between downloading multiple albums')
  parser.add_argument('--format', dest='foldername_format', action='store', default='%t',
                      help='format album folder name')

  args = parser.parse_args()

  return args


if __name__ == '__main__':
  command_line()
